from typing import Generic, TypeVar

from rainbowadn.flow.core import *
from ._compositiondispatch import *
from ._sequencedispatch import *
from ._sequencedispatcher import *

__all__ = ('CompositionDispatcher',)

Element = TypeVar('Element')
Out = TypeVar('Out')
Middle = TypeVar('Middle')


class CompositionDispatcher(
    SequenceDispatcher[Middle, Out],
    Generic[Middle, Out, Element]
):
    def __init__(
            self,
            mapper: Mapper[Element, Middle],
            dispatcher: SequenceDispatcher[Element, Out]
    ):
        assert isinstance(mapper, Mapper)
        assert isinstance(dispatcher, SequenceDispatcher)
        self.mapper = mapper
        self.dispatcher = dispatcher

    async def dispatch(self, dispatch: SequenceDispatch[Middle, Out]) -> Out:
        cd: SequenceDispatch[Element, Out] = CompositionDispatch(self.mapper, dispatch)
        assert isinstance(cd, SequenceDispatch)
        return await self.dispatcher.dispatch(
            cd
        )

    @classmethod
    def mapper(cls, mapper: Mapper[Element, Middle]) -> Mapper[
        SequenceDispatcher[Element, Out],
        SequenceDispatcher[Middle, Out],
    ]:
        assert isinstance(mapper, Mapper)

        def wrap(dispatcher: SequenceDispatcher[Element, Out]) -> SequenceDispatcher[Middle, Out]:
            assert isinstance(dispatcher, SequenceDispatcher)
            return CompositionDispatcher(
                mapper,
                dispatcher
            )

        return CallableMapper(wrap)
