from typing import Generic, TypeVar

from ._sequencedispatch import *

__all__ = ('SequenceDispatcher', 'FirstDispatcher', 'LastDispatcher', 'PairDispatcher',)

Element = TypeVar('Element')
Out = TypeVar('Out')


class SequenceDispatcher(Generic[Element, Out]):
    async def dispatch(self, dispatch: SequenceDispatch[Element, Out]) -> Out:
        raise NotImplementedError


class FirstDispatcher(SequenceDispatcher[Element, Out]):
    def __init__(self, element: Element):
        self.element = element

    async def dispatch(self, dispatch: SequenceDispatch[Element, Out]) -> Out:
        assert isinstance(dispatch, SequenceDispatch)
        return await dispatch.on_first(self.element)


class LastDispatcher(SequenceDispatcher[Element, Out]):
    def __init__(self, element: Element):
        self.element = element

    async def dispatch(self, dispatch: SequenceDispatch[Element, Out]) -> Out:
        assert isinstance(dispatch, SequenceDispatch)
        return await dispatch.on_last(self.element)


class PairDispatcher(SequenceDispatcher[Element, Out]):
    def __init__(self, previous: Element, element: Element):
        self.previous = previous
        self.element = element

    async def dispatch(self, dispatch: SequenceDispatch[Element, Out]) -> Out:
        assert isinstance(dispatch, SequenceDispatch)
        return await dispatch.on_pair(self.previous, self.element)
